using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using Microsoft.StylusInput;
using Microsoft.StylusInput.PluginData;
using Microsoft.Ink;
using System.Runtime.InteropServices;

/// Kenrick Mock
/// Mouse attention-highlighter

namespace PenAttention
{
    // The class must implement the IStylusAsyncPlugin interface
    // to get pen notification calls
    public partial class frmAttention : Form, IStylusAsyncPlugin
    {
        private RealTimeStylus myRealTimeStylus;    // Used to detect pen
        public static bool pen_detected = false;    // Whether or not pen is nearby
        public enum Attention_Mode { Highlight, Pencil, Pointer, None };
        private static Attention_Mode attention_type = Attention_Mode.Highlight;
        long lMsg;
        IntPtr myHwnd;

        [DllImport("user32.dll", SetLastError = true)]
        static extern bool ChangeWindowMessageFilter(uint message, uint dwFlag);

        [DllImport("user32.dll")]
        static extern int SetWindowLong(IntPtr hWnd, int nIndex, int dwNewLong);

        [DllImport("user32.dll")]
        static extern long GetWindowLong(IntPtr hWnd, int nIndex);

        [DllImport("MouseHook.dll")]
        static extern bool setMyHook(IntPtr hWnd);

        [DllImport("MouseHook.dll")]
        static extern bool clearMyHook(IntPtr hWnd);

        [DllImport("MouseHook.dll")]
        static extern void setFlag(int flag);

        [DllImport("user32.dll", EntryPoint = "FindWindow", SetLastError = true)]
        static extern System.IntPtr FindWindowByCaption(int ZeroOnly, string lpWindowName);

        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        static extern uint RegisterWindowMessage(string lpString);

        public frmAttention()
        {
            InitializeComponent();
            myRealTimeStylus = new RealTimeStylus(this.pboxPencil, false); // Register stylus, not mouse
            myRealTimeStylus.AsyncPluginCollection.Add(this);   // Get callbacks on pen activity
            myRealTimeStylus.Enabled = true;
            pen_detected = false;
        }

        private void frmSettings_Load(object sender, EventArgs e)
        {
            SetWindowLong(this.Handle, -20, (int)GetWindowLong(this.Handle, -20) | 0x00000020);
            this.Width = 30;
            this.Height = 30;

            this.Left = System.Windows.Forms.Control.MousePosition.X -15;
            this.Top = System.Windows.Forms.Control.MousePosition.Y - 15;

            myHwnd = FindWindowByCaption(0, this.Text);
            setMyHook(myHwnd);
            //Register the message
            lMsg = RegisterWindowMessage("UWM_MOUSEMOVE_MSG-PENATTEN-44E531B1_14D3_11d5_A025_006067718D04");


            int osversion = System.Environment.OSVersion.Version.Major;
            if (osversion >= 6)
            {
                // Vista, add message to filter list so we can get it from IE
                ChangeWindowMessageFilter(Convert.ToUInt32(lMsg), 1);
            }


            int attentionMode = PenAttention.Properties.Settings.Default.selectedIcon;
            if (attentionMode == 0)
            {
                attention_type = Attention_Mode.Highlight;
                this.showHighlightToolStripMenuItem.Checked = true;
            }
            else if (attentionMode == 1)
            {
                attention_type = Attention_Mode.Pencil;
                this.showPencilToolStripMenuItem.Checked = true;
            }
            else if (attentionMode == 2)
            {
                attention_type = Attention_Mode.Pointer;
                this.showPointerToolStripMenuItem.Checked = true;
            }
            else if (attentionMode == -1)
            {
                attention_type = Attention_Mode.None;
            }
            //Hide();
        }

        public void MouseMoved(int X, int Y)
        {
            // If no pen, do normal mouse thing
            if (!pen_detected)
            {
                return;
            }

            // If no attention mode, do nothing
            if (attention_type == Attention_Mode.None)
            {
                return;
            }

            // Pen detected so draw attention to the mouse location appropriately
            if (attention_type == Attention_Mode.Highlight)
            {
                this.Left = X - 15;
                this.Top = Y - 15;
            }
            else
            {
                this.Left = X;
                this.Top = Y;
            }
        }

        protected override void WndProc(ref Message m)
        {
            //calling the base first is important, otherwise the values you set later will be lost
            base.WndProc(ref m);

            if (m.Msg == lMsg)
            {
                MouseMoved(m.WParam.ToInt32(), m.LParam.ToInt32());
                //Console.WriteLine(m.WParam + " " + m.LParam + ": " + System.Windows.Forms.Cursor.Position.X + "," + System.Windows.Forms.Cursor.Position.Y);
            }
        }

        private void frmSettings_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Uninitialize hooks
            //actHook.Stop();
            clearMyHook(myHwnd);
        }

        private void btnHide_Click(object sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Minimized;
            Hide();
        }

        #region IStylusAsyncPlugin interface implementation

        /// <summary>
        /// Informs the implementing object that user data is available.
        /// </summary>
        /// <param name="sender">The real time stylus associated with the notification</param>
        /// <param name="data">The notification data</param>
        public void CustomStylusDataAdded(RealTimeStylus sender, CustomStylusData data)
        {
        }

        /// <summary>
        /// Called when the current plugin or the ones previous in the list
        /// threw an exception.
        /// </summary>
        /// <param name="sender">The real time stylus associated with the notification</param>
        /// <param name="data">The notification data</param>
        public void Error(RealTimeStylus sender, ErrorData data)
        {
            Debug.Assert(false, null, "An error occurred.  DataId=" + data.DataId + ", " + "Exception=" + data.InnerException);
        }

        /// <summary>
        /// Defines the types of notifications the plugin is interested in.
        /// </summary>
        public DataInterestMask DataInterest
        {
            get
            {
                return DataInterestMask.StylusDown | DataInterestMask.StylusUp | DataInterestMask.StylusOutOfRange | DataInterestMask.StylusInRange | DataInterestMask.Error;
                //return DataInterestMask.StylusDown | DataInterestMask.StylusUp |  DataInterestMask.Error;
            }
        }

        public void StylusOutOfRange(RealTimeStylus sender, StylusOutOfRangeData data)
        {
            //Console.WriteLine("Outta range");
            pen_detected = false;
            setFlag(1);
            this.Invalidate();
        }
        public void StylusInRange(RealTimeStylus sender, StylusInRangeData data)
        {
            //Console.WriteLine("Pen in range");
            pen_detected = true;
            setFlag(0);
            this.Invalidate();
        }

        public void StylusDown(RealTimeStylus sender, StylusDownData data) 
        {
            Console.WriteLine("Down");
            pen_detected = false;
            this.Invalidate();
        }

        public void StylusUp(RealTimeStylus sender, StylusUpData data) 
        {
            Console.WriteLine("Up");
            pen_detected = true;
            this.Invalidate();
        }


        // The remaining interface methods are not used in this sample application.
        public void RealTimeStylusDisabled(RealTimeStylus sender, RealTimeStylusDisabledData data) { }
        public void RealTimeStylusEnabled(RealTimeStylus sender, RealTimeStylusEnabledData data) { }
        public void StylusButtonDown(RealTimeStylus sender, StylusButtonDownData data) { }
        public void StylusButtonUp(RealTimeStylus sender, StylusButtonUpData data) { }
        public void SystemGesture(RealTimeStylus sender, SystemGestureData data) { }
        public void Packets(RealTimeStylus sender, PacketsData data) { }
        public void InAirPackets(RealTimeStylus sender, InAirPacketsData data) { }
        public void TabletAdded(RealTimeStylus sender, TabletAddedData data) { }
        public void TabletRemoved(RealTimeStylus sender, TabletRemovedData data) { }
        #endregion

        private void frmSettings_Resize(object sender, EventArgs e)
        {
            if (FormWindowState.Minimized == WindowState)
                Hide();
        }

        private void notifyIcon1_DoubleClick(object sender, EventArgs e)
        {
            contextMenuStrip1.Show();
            //Show();
            //WindowState = FormWindowState.Normal;
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private bool nothingChecked()
        {
            if (!showHighlightToolStripMenuItem.Checked && !showPencilToolStripMenuItem.Checked && !showPointerToolStripMenuItem.Checked)
                return true;
            return false;
        }

        private void showSettingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (showHighlightToolStripMenuItem.Checked)
            {
                // Undo other checkmarks
                showPencilToolStripMenuItem.Checked = false;
                showPointerToolStripMenuItem.Checked = false;

                // Highlight the pen option
                Show();
                WindowState = FormWindowState.Normal;
                this.Opacity = 0.4;

                attention_type = Attention_Mode.Highlight;
                PenAttention.Properties.Settings.Default.selectedIcon = 0;
                PenAttention.Properties.Settings.Default.Save();
                this.Invalidate();
            }
            else
            {
                if (nothingChecked())
                {
                    attention_type = Attention_Mode.None;
                    PenAttention.Properties.Settings.Default.selectedIcon = -1;
                    PenAttention.Properties.Settings.Default.Save();
                    Hide();
                }
            }
        }

        private void showPencilToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (showPencilToolStripMenuItem.Checked)
            {
                // Undo other checkmarks
                showHighlightToolStripMenuItem.Checked = false;
                showPointerToolStripMenuItem.Checked = false;

                // Show Pencil option
                Show();
                WindowState = FormWindowState.Normal;
                this.Opacity = 0.99;
                attention_type = Attention_Mode.Pencil;
                PenAttention.Properties.Settings.Default.selectedIcon = 1;
                PenAttention.Properties.Settings.Default.Save();
                this.Invalidate();
            }
            else
            {
                if (nothingChecked())
                {
                    attention_type = Attention_Mode.None;
                    PenAttention.Properties.Settings.Default.selectedIcon = -1;
                    PenAttention.Properties.Settings.Default.Save();
                    Hide();
                }
            }
        }

        private void showPointerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (showPointerToolStripMenuItem.Checked)
            {
                // Undo other checkmarks
                showHighlightToolStripMenuItem.Checked = false;
                showPencilToolStripMenuItem.Checked = false;

                // Show pointer option
                Show();
                WindowState = FormWindowState.Normal;
                this.Opacity = 0.99;
                attention_type = Attention_Mode.Pointer;
                PenAttention.Properties.Settings.Default.selectedIcon = 2;
                PenAttention.Properties.Settings.Default.Save();
                this.Invalidate();
            }
            else
            {
                if (nothingChecked())
                {
                    attention_type = Attention_Mode.None;
                    PenAttention.Properties.Settings.Default.selectedIcon = -1;
                    PenAttention.Properties.Settings.Default.Save();
                    Hide();
                }
            }
        }

        private void frmAttention_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            // If no attention mode, or no pen, do nothing
            if (attention_type == Attention_Mode.None || pen_detected == false)
            {
                return;
            }

            if (attention_type == Attention_Mode.Highlight)
            {
                g.FillEllipse(Brushes.Salmon, 0, 0, this.Width - 1, this.Height - 1);
            }
            else if (attention_type == Attention_Mode.Pencil)
            {
                g.DrawImage(pboxPencil.Image, 0, 0);
            }
            else if (attention_type == Attention_Mode.Pointer)
            {
                g.DrawImage(pboxPointer.Image, 0, 0);
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmAbout about = new frmAbout();
            about.ShowDialog();
        }

        private void notifyIcon1_Click(object sender, EventArgs e)
        {
            int x  = System.Windows.Forms.Cursor.Position.X;
            int y = System.Windows.Forms.Cursor.Position.Y - 40;
            contextMenuStrip1.Show( x, y-100);
        }

    }
}